@@ -29,7 +29,7 @@ Environment Variables
 ``CELERY_RDB_HOST``
 -------------------
 
-    Hostname to bind to.  Default is '127.0.01' (only accessable from
+    Hostname to bind to.  Default is '127.0.0.1' (only accessable from
     localhost).
 
 .. envvar:: CELERY_RDB_PORT
@@ -53,10 +53,10 @@ from billiard.process import current_process
 
 from celery.five import range
 
-__all__ = [
+__all__ = (
     'CELERY_RDB_HOST', 'CELERY_RDB_PORT', 'DEFAULT_PORT',
     'Rdb', 'debugger', 'set_trace',
-]
+)
 
 DEFAULT_PORT = 6899
 
@@ -127,6 +127,7 @@ class Rdb(Pdb):
         this_port = None
         for i in range(search_limit):
             _sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            _sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
             this_port = port + skew + i
             try:
                 _sock.bind((host, this_port))
