@@ -18,7 +18,7 @@ from __future__ import absolute_import, unicode_literals
 import bisect
 import sys
 import threading
-from collections import Callable, defaultdict
+from collections import defaultdict
 from datetime import datetime
 from decimal import Decimal
 from itertools import islice
@@ -34,7 +34,14 @@ from celery.five import items, python_2_unicode_compatible, values
 from celery.utils.functional import LRUCache, memoize, pass1
 from celery.utils.log import get_logger
 
-__all__ = ['Worker', 'Task', 'State', 'heartbeat_expires']
+try:
+    from collections.abc import Callable
+except ImportError:
+    # TODO: Remove this when we drop Python 2.7 support
+    from collections import Callable
+
+
+__all__ = ('Worker', 'Task', 'State', 'heartbeat_expires')
 
 # pylint: disable=redefined-outer-name
 # We cache globals and attribute lookups, so disable this warning.
@@ -280,7 +287,7 @@ class Task(object):
     merge_rules = {
         states.RECEIVED: (
             'name', 'args', 'kwargs', 'parent_id',
-            'root_id' 'retries', 'eta', 'expires',
+            'root_id', 'retries', 'eta', 'expires',
         ),
     }
 
